odf_function=function (name,fun) 
{
  structure(fun, name = name)
}


# Zero Crossing Rate
ZerroCrossingRate=function(fa,ctrl)
{
  A=fa$ampl
  A_mov=rbind(A[-1,],rep(0,ncol(A)))
  AA=A*A_mov
  ZCR=apply(AA,2,function(x)sum(x<0))
  return(ZCR)
}


ZerroCrossingRateDiff=function(fa, ctrl){
  ZCR=ZerroCrossingRate(fa, ctrl)
  ZCR_1diff=abs(diff(c(0,ZCR)))
  attr(ZerroCrossingRateDiff,"name")="ZCR.D"
  return(ZCR_1diff)
}

ZerroCrossingRateDiff=odf_function("ZCRate",ZerroCrossingRateDiff)


# Amplitude maximum difference
AmplitMaxDiff=function(fa,ctrl) {
  Max.of.wind=apply(fa$ampl, 2, function(x) max(abs(x)))
  AMD=diff(c(0, Max.of.wind))
  return(AMD)
}
AmplitMaxDiff=odf_function("AMax",AmplitMaxDiff)


# Amplitude maximum absolute difference
AmplitMaxAbsDiff=function(fa,ctrl) {
  Max.of.wind=apply(fa$ampl, 2, function(x) max(abs(x)))
  AMD=abs(diff(c(0, Max.of.wind)))
  return(AMD)
}
AmplitMaxAbsDiff=odf_function("AMaxA",AmplitMaxAbsDiff)


# Amplitude energy
AmplitEnergy=function(fa,ctrl) {
  AE=apply(fa$ampl, 2, function(x) sum(x^2))
  return(AE)
}

# Amplitude energy difference
AmplitEnergyDiff=function(fa,ctrl) {
  AE=AmplitEnergy(fa,ctrl)
  AED=diff(c(0, AE)) 
  return(AED)
}
AmplitEnergyDiff=odf_function("AEnerg",AmplitEnergyDiff)


# Amplitude energy absolute difference
AmplitEnergyAbsDiff=function(fa,ctrl) {
  AE=AmplitEnergy(fa,ctrl)
  AED=diff(c(0, AE)) 
  return(abs(AED))
}
AmplitEnergyAbsDiff=odf_function("AEnergA",AmplitEnergyAbsDiff)




# Spectral energy
SpecEnergy=function(fa, ctrl) {
  S = fa$spec
  N=dim(S)[1]
  #S=cbind(rep(0,N), S, rep(0, N)) 
  SE=apply(S, 2, function(x) sum(abs(x)^2))/N
  return(SE)
}

# Spectral energy difference
SpecEnergyDiff=function(fa, ctrl) {
  SE=SpecEnergy(fa, ctrl)
  SE.D=diff(c(0, SE)) 

  return(SE.D)
}
SpecEnergyDiff=odf_function("SEnerg",SpecEnergyDiff)


# Spectral energy absolute difference
SpecEnergyAbsDiff=function(fa, ctrl) {
  SE.D=SpecEnergyDiff(fa, ctrl)
  return(abs(SE.D))
}
SpecEnergyAbsDiff=odf_function("SEnergA",SpecEnergyAbsDiff)



# High frequency content
HighFreqCont=function(fa, ctrl) {
  S = fa$spec
  N=dim(S)[1]
 # S=cbind(rep(0,N), S, rep(0, N)) 
  k=1:N
  HFC=apply(S, 2, function(x) sum(k*abs(x)^2))/N
  return(HFC)
}

# High frequency content difference
HighFreqContDiff=function(fa, ctrl) {
  HFC=HighFreqCont(fa, ctrl)
  HFC.D=diff(c(0, HFC)) 
  return(HFC.D)
}
HighFreqContDiff=odf_function("HFCont",HighFreqContDiff)


# High frequency content absolute difference
HighFreqContAbsDiff=function(fa, ctrl) {
  HFC.D=HighFreqContDiff(fa, ctrl) 
  return(abs(HFC.D))
}
HighFreqContAbsDiff=odf_function("HFContA",HighFreqContAbsDiff)


# Gauss frequency content
HighFreqContGaus=function(fa, ctrl) {
  S = fa$spec
  N=dim(S)[1]
  k=gausswin(N)
  HFCG=apply(S, 2, function(x) sum(k*abs(x)^2))/N
  return(HFCG)
}


# Gauss frequency content difference
HighFreqContGausDiff=function(fa, ctrl) {
  HFCG.D=HighFreqContGaus(fa, ctrl)
  return(diff(c(0,HFCG.D)))
}
HighFreqContGausDiff=odf_function("GFCont",HighFreqContGausDiff)


# Gauss frequency content absolute difference
HighFreqContGausAbsDiff=function(fa, ctrl) {
  HFCG.D=HighFreqContGausDiff(fa, ctrl)
  return(abs(HFCG.D))
}
HighFreqContGausAbsDiff=odf_function("GFContA",HighFreqContGausAbsDiff)



# Spectral centroid
SpecCentroid=function(fa, ctrl){
  S = fa$spec
  k= dim(S)[1] # nr. of frequency bins
  upp = apply(S,2,function(x) {sum(abs(x)*(1:k))})
  low = apply(S,2,function(x) {sum(abs(x))})
  SC = upp / low
  return(SC)
}

# Spectral centroid differenz
SpecCentroidDiff=function(fa, ctrl){
  SC=SpecCentroid(fa, ctrl)
  SC_1diff=abs(diff(c(0,SC)))
  return(SC_1diff)
}
SpecCentroidDiff=odf_function("SCentr",SpecCentroidDiff)


# Spectral spread
SpecSpread=function(fa, ctrl){
  S = fa$spec
  k = dim(S)[1] # nr. of frequency bins
  m = dim(S)[2] # nr. of windows
  SC=SpecCentroid(fa, ctrl)
  
  SS=numeric(m)
  for (i in 1:m) {
    abs.S=abs(S[,i])
    SS[i]=sqrt( sum(abs.S*((1:k)-SC[i])^2)) /
      sqrt(sum(abs.S))
  }
return(SS)
}


# Spectral spread differenz
SpecSpreadDiff=function(fa, ctrl){
  SSp=SpecSpread(fa, ctrl)
  SSp_1diff=abs(diff(c(0,SSp)))
  return(SSp_1diff)
}
SpecSpreadDiff=odf_function("SSpread",SpecSpreadDiff)


# Spectral skewness
SpecSkewness=function(fa, ctrl){
  S = fa$spec
  k = dim(S)[1] # nr. of frequency bins
  m = dim(S)[2] # nr. of windows
  SC=SpecCentroid(fa, ctrl)
  SSp=SpecSpread(fa, ctrl)
  
  SSk=numeric(m)
  for (i in 1:m) {
    abs.S=abs(S[,i])
    SSk[i]=sum(abs.S*((1:k)-SC[i])^3) /
      (SSp[i]^3*(sum(abs.S)))
  }
  return(SSk)
}

# Spectral spread difference
SpecSkewnessDiff=function(fa, ctrl){
  SSk=SpecSkewness(fa, ctrl)
  SSk_1diff=abs(diff(c(0,SSk)))
  return(SSk_1diff)
}

SpecSkewnessDiff=odf_function("SSkew",SpecSkewnessDiff)



# Spectral Flux
SpecFlux=function(fa, ctrls) 
{
  S=fa$spec
  n=dim(S)[2]
  H=function(x){
    (x+abs(x))/2
	}
  SF=numeric(n)
  #SF[1]=sum(H(abs(S[,1]))) # erstes Element bei 0 lassen!
  for (i in 2:n){
    SF[i]=sum( H(abs(S[,i])-abs(S[,(i-1)])) ) 
  }
  return(SF) # damit feauture vector genauso viele Eintr?ge hat wie #Fenster
}
SpecFlux=odf_function("SFlux",SpecFlux)


# Spectral euclid distance
SpecFluxEuclid=function(fa, ctrls)
{
  S=fa$spec
  n=dim(S)[2]
  
  H=function(x){
    x^2
  }
  SF=numeric(n)
  #SF[1]=sum(H(abs(S[,1])))
  for (i in 2:n)
  {
    SF[i]=sum( H(abs(S[,i])-abs(S[,(i-1)])) ) 
  }
  return(SF)
}
SpecFluxEuclid=odf_function("SFluxEucl",SpecFluxEuclid)


# Phase deviation
PhaseDeviation=function(fa, ctrl) 
{
 
  # intialization
  phi=fa$phase
  n=dim(phi)[2]  # number of windows
  N=dim(phi)[1]
  phi=cbind(rep(0,N),rep(0,N),phi)
  
  phi_2=phi
  PD=numeric(n+2)
  
  for (i in 3:(n+2)) {
    #phi_1[,i]=phi[,i]-phi[,(i-1)]
    phi_2[,i]=phi[,i]-2*phi[,(i-1)]+phi[,(i-2)]
    PD[i]=(1/N)*sum(abs(phi_2[,i]))
  }
  return(PD[-(1:2)])
}
PhaseDeviation=odf_function("PDev",PhaseDeviation)


# Normalized Weighted phase deviation
WeightPhaseDeviation=function(fa, ctrl) {
  
  # intialization
  phi=fa$phase
  n=dim(phi)[2]  # number of windows
  N=dim(phi)[1]
  phi=cbind(rep(0,N),rep(0,N),phi)
  
  phi_2=phi
  WPD=numeric(n+2)
  
   
  S = fa$spec.compl
  S=cbind(rep(0,N),rep(0,N),S)
   
  for (i in 3:(n+2)) # for all windows
  {
    #phi_1[,i]=phi[,i]-phi[,(i-1)]
    phi_2[,i]=phi[,i]-2*phi[,(i-1)]+phi[,(i-2)]
    WPD[i]=(1/N)*sum(abs(S[,i]*phi_2[,i]))
  }
  return(WPD[-(1:2)])
}
WeightPhaseDeviation=odf_function("WPDev",WeightPhaseDeviation)


# Complex Domain
ComplexDomain=function(fa, ctrl) {
   S = fa$spec.compl
   n=dim(S)[2]  # number of windows
   N=dim(S)[1] # number of samples 
   S=cbind(rep(0,N),rep(0,N),S)
   S.estim=S
  # intialization
  
  phi=fa$phase
  phi=cbind(rep(0,N),rep(0,N),phi)
  
  phi.est=phi
  CD=numeric(n+2)
  
  for (i in 3:(n+2)) {
    phi.est[,i]=2*phi[,(i-1)]-phi[,(i-2)]
    S.estim[,i]=complex(modulus = Mod(S[,(i-1)]), argument = phi.est[,i])
    CD[i]=(1/N)*sum(abs(S[,i]-S.estim[,i]))
  }
  return(CD[-(1:2)])
}
ComplexDomain=odf_function("CDom",ComplexDomain)


# Rectified Complex Domain
RectComplexDomain=function(fa, ctrl) {
  S = fa$spec.compl
  n=dim(S)[2]  # number of windows
  N=dim(S)[1] # number of samples
  S=cbind(rep(0,N),rep(0,N),S)
  S.estim=S
  # intialization
  phi=fa$phase
  phi=cbind(rep(0,N),rep(0,N),phi)
  
  phi.est=phi
  RCD=numeric(n+2)
  
  
    for (i in 3:(n+2)){
    phi.est[,i]=2*phi[,(i-1)]-phi[,(i-2)]
    S.estim[,i]=complex(modulus = Mod(S[,(i-1)]), argument = phi.est[,i])
  }
  
  a.S = abs(S)-abs(S.estim)
  w.pos = a.S > 0
  A = abs(S-S.estim) * w.pos
  RCD = apply(A, 2, sum) * (1/dim(S)[1])
  return(RCD[-(1:2)])
}
RectComplexDomain=odf_function("RCDom",RectComplexDomain)


getODF=function(ctrl){
  switch(ctrl$odf,
         ZCRate=ZerroCrossingRateDiff,
         AMax=AmplitMaxDiff,
         AMaxA=AmplitMaxAbsDiff,
         AEnerg=AmplitEnergyDiff,
         AEnergA=AmplitEnergyAbsDiff,
         SEenerg=SpecEnergyDiff,
         SEenergA=SpecEnergyAbsDiff,
         HFCont=HighFreqContDiff,
         HFContA=HighFreqContAbsDiff,
         GFCont=HighFreqContGausDiff,
         GFContA=HighFreqContGausAbsDiff,
         SCentr=SpecCentroidDiff,
         SSpread=SpecSpreadDiff,
         SSkew=SpecSkewnessDiff,
         SFlux=SpecFlux,
         SFluxEucl=SpecFluxEuclid,
         PDev=PhaseDeviation,
         WPDev=WeightPhaseDeviation,
         CDom=ComplexDomain,
         RCDom=RectComplexDomain,
         # default: try to match the fun which is given as string
         match.fun(ctrl$odf)
  )
}